//
// C++ Implementation: binarygraph
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qlabel.h>
#include <qpainter.h>
#if QT_VERSION >= 0x40000
#include <QMouseEvent>
#endif

#include "commobj.h"
#include "binarygraph.h"
#include "ar3cmonitor.h"
#include "consts.h"
#include "control.h"

BinaryGraph::BinaryGraph(int _physical, const QColor &_acolor,
		const QColor &_icolor, const QString &_symbol,
		QWidget *parent, const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f)
{
	setMinimumHeight(QFontMetrics(CO->smallFont()).height());
	setMinimumWidth(100);

	acolor = _acolor;
	icolor = _icolor;
	symbol = _symbol;
	physical = _physical;
	old_pos = -1;
}

void BinaryGraph::mousePressEvent(QMouseEvent *e)
{
	if (CO->reviewMode())
		old_pos = e->x();
}

void BinaryGraph::mouseMoveEvent(QMouseEvent *e)
{
	if (old_pos != -1) {
		int pos = e->x();
		int w = width();

		BUG_ON(!w);

		CO->advanceCursor((old_pos - pos) * (long long) CO->timeRange()	/ w);
		old_pos = pos;
	}
}

void BinaryGraph::mouseReleaseEvent(QMouseEvent *)
{
	old_pos = -1;
}

void BinaryGraph::paint(QPainter *p)
{
	int y = height() / 2;
	int w = width() - QFontMetrics(CO->smallFont()).width("WW");

	p->setPen(acolor);
	p->setFont(CO->smallFont());
	p->drawText(w + 2, 0, width() - w - 2, height(),
		Qt::AlignLeft | Qt::AlignVCenter, symbol);

	if (!CO->haveData()) {
		p->setPen(QPen(icolor, 1, Qt::DashLine));
		p->drawLine(0, y, w, y);
		return;
	}

	unsigned long long T1 = CO->cursorStamp();
	unsigned long long T0 = T1 - CO->timeRange();

	if (CO->vgridStep()) {
		p->setPen(CO->vgridColor());

		unsigned long long step = CO->vgridStep();
		for (unsigned long long T = T0 + step - T0 % step;
			T <= T1; T += step) {
			int x = calculateX(T, T0, w, CO->timeRange());

			p->drawLine(x, y - 2, x, y + 2);
		}
	}

	p->setPen(QPen(icolor, 1));
	p->drawLine(0, y, w, y);

	int x0 = w;

	TimestampedList < BRecord >::Iterator i =
			CO->binary(physical)->Cursor.scanLeftBefore(T1);
	for (; i.valid(); --i) {
		unsigned long long ts = i.timeStamp();
		int x = calculateX_bin(ts, T0, w, CO->timeRange());
		BState s = i.data()->state;

		if (s == Undefined) {
			p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv), 3));
			p->drawLine(x0, y, x == x0 ? x0 + 1 : x, y);
		} else if (s == High) {
			p->setPen(QPen(acolor, 3));
			p->drawLine(x0, y, x == x0 ? x0 + 1 : x, y);
		}
		x0 = x;
		if (ts < T0)
			break;
	}
	if (x0 > 0) {
		p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv), 3));
		p->drawLine(0, y, x0, y);
	}
	TimestampedList < ARecord >::Iterator it = CO->analog()->Cursor;
	if (!it.valid() || it.timeStamp() < CO->cursorStamp()) {
		int x = calculateX_bin(it.timeStamp(), T0, w, CO->timeRange());
		p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv), 3));
		p->drawLine(w, y, x + 1, y);
	}
}
