//
// C++ Interface: commobj
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005, 2006, 2007  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qglobal.h>
#include <qtimer.h>
#include <qwidget.h>
#include <token.h>

#include "ar3cm_proto.h"

#if QT_VERSION >= 0x40000
# include <Q3Socket>
# define QSocket Q3Socket
#else
# include <qsocket.h>
#endif

#if QT_VERSION >= 0x30000
# define QSOCK_CONNECTED	 QSocket::Connected
#else
# define QSOCK_CONNECTED	 QSocket::Connection
#endif

class Ar3cMonitor;
class Action;

enum token_state {
	TOKEN_NONE,	/* no token */
	TOKEN_WANT,	/* token wanted, get_token send */
	TOKEN_READY,	/* token received */
};

class CommObject : public QObject {
	Q_OBJECT
private:
	QSocket *Socket;
	QTimer *Watchdog;
	QTimer *FullRequestTimer;
	QTimer *reconnect_timer;
	Ar3cMonitor *Owner;
	struct ar3cm_proto_hdr PacketHdr;
	bool haveData;
	unsigned long long lastTimestamp;
	unsigned char *SetupSeq;
	long SetupSeqOff;
	bool offline;
	bool FullyConnected;
	unsigned int orientation;
	
	unsigned char *password;
	size_t password_size;

	enum token_state token_state;
	struct token token;

	void parsePacket(struct ar3cm_proto_hdr *hdr, unsigned char *buf);
	void parse_binitem(struct ar3cm_proto_hdr *hdr, unsigned char *buf);
	Action *parseAction(struct ar3cm_proto_action *a);
	void sendSimpleCommand(int opcode);
	void sendCommand(int opcode, const void *buf, int len);
	void handleSetupSeq(unsigned char *buf);
	void startOfflineProtocol();
	int check_packet_size(struct ar3cm_proto_hdr *hdr, size_t min,
			size_t max);
	int check_packet_size_min(struct ar3cm_proto_hdr *hdr, size_t min);
	int check_packet_size_eq(struct ar3cm_proto_hdr *hdr, size_t size);
	QColor hsv_qcolor(struct ar3cm_proto_hsv *c);
	void receive_token(struct ar3cm_proto_hdr *hdr,
			struct ar3cm_proto_token *t);

private slots:
	void connectedHandler();
	void closedHandler();
	void readHandler();
	void watchdogHandler();
	void requestData();

public:
	CommObject(Ar3cMonitor * owner);
	~CommObject();

	bool canGoOffline();
	void requestMinute(unsigned long min);
	void request_locale(const char *locale);
	void requestScheme(unsigned int scheme);
	void set_opt(unsigned long opt, unsigned long value);
	bool fullyConnected()
	{
		return FullyConnected;
	}
	bool reallyConnected();
	int setDataloggerTime(unsigned long long ts, bool force_imidiate);
	void get_token();
	int set_password(const char *pass);
	int pendrive_copy(unsigned long long from,unsigned long long to,
			int format);

public slots:
	void errorHandler(int err);
	void connectToAr3c();
	void goOnline();
	void goOffline();

signals:
	void sigFullyConnected();
	void sigReallyConnected();
	void sigOfflineConnected();
	void sigConnected();
	void sigConnecting();
	void sigError();
	void sig_pendrive_error(unsigned int number);
	void sig_pendrive_complete(unsigned int status);
	void sig_pendrive_insert();
	void sig_pendrive_eject();
	void sig_pendrive_progress(unsigned int progress);
};

extern CommObject *TheCommObject;
