//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef AR3CMON_CONSTS_H_INCLUDED
#define AR3CMON_CONSTS_H_INCLUDED

#ifdef __GNUC__

#define likely(x)	__builtin_expect(!!(x), 1)
#define unlikely(x)	__builtin_expect(!!(x), 0)

#else

#define __attribute__(x)		/* unsupported */

#define likely(x)	(x)
#define unlikely(x)	(x)

#endif

#include <qwidget.h>
#ifndef Q_ASSERT

#ifndef NDEBUG
#include <stdio.h>

#if defined __cplusplus ? __GNUC_PREREQ (2, 6) : __GNUC_PREREQ (2, 4)
#  define __QASSERT_FUNCTION	__PRETTY_FUNCTION__
#else
# if defined __STDC_VERSION__ && __STDC_VERSION__ >= 199901L
#  define __QASSERT_FUNCTION	__func__
# else
#  define __QASSERT_FUNCTION	((__const char *) 0)
# endif
#endif

#define Q_ASSERT(cond)							\
	do {								\
		if (!(cond)) {						\
			fprintf(stderr, "%s:%u: %s%s Assertion '%s' "	\
					"failed\n", 			\
			__FILE__, __LINE__,				\
			__QASSERT_FUNCTION ? __QASSERT_FUNCTION : "",	\
			__QASSERT_FUNCTION ? ": " : "",			\
			__STRING(cond));				\
		}							\
	} while (0)
#else
#define Q_ASSERT(x...)	do { } while (0)
#endif
#endif

#ifndef Q_CHECK_PTR
#define Q_CHECK_PTR(x)	Q_ASSERT((x) != NULL)
#endif

#define AR3CMON_MAX_ANALOG_INPUTS	256
#define AR3CMON_MAX_BINARY_INPUTS	4096
#define AR3CMON_MAX_TABS		256

#if QT_VERSION < 0x30000
#define Q_LONG long
#define trUtf8(x) tr(x)
#endif

#define kB_B(x)		((x) * 1024UL)
#define MB_B(x)		((x) * 1048576UL)

#if defined(__SIMPAD__)
#include "config-simpad.h"
#elif defined(__LCD57__)
#include "config-lcd57.h"
#elif defined(__WIN32__)
#include "config-win32.h"
#elif defined(USER_CONFIG)
#include "config-user.h"
#else
#include "config-default.h"
#endif
#include "config-generic.h"

#define AR3CMON_COMM_BUF_SIZE		4096
#define MREQ_LIMIT_DEFAULT		8

#define AR3CMON_CONNECTION_TIMEOUT	5000
#define AR3CMON_COMMUNICATION_TIMEOUT	5000
#define AR3CMON_DATA_TIMEOUT		1000

#define AR3CMON_MAX_DZWONEK_SERIA	15000000000ULL	/* 15s */
#define AR3CMON_MAX_DZWONEK_ODSTEP	1000000000ULL	/* 1s */

#ifndef AR3CMON_DFLT_HOST
#define AR3CMON_DFLT_HOST	"127.0.0.1"
#endif

#define AR3CMON_DFLT_PORT	2346

#ifndef __GLIBC__
#warning "You should specify new and malloc overhead for your system"
#endif

/* alignments must be 2^n */
#define AR3CMON_NEW_OVERHEAD		16	/* new[] overhead */
#define AR3CMON_NEW_ALIGNMENT		8	/* new[] alignment */

#ifdef HAVE_BALLOC
#include <balloc.h>

#define mon_alloc(size)		b_malloc(size)
#define mon_free(x)		b_free(x)

/* real overhead is ab. 2 bits */
#define MONALLOC_OVERHEAD		1	/* mon_alloc() overhead */
#define MONALLOC_ALIGNMENT		8	/* mon_alloc() alignment */
#else
#define mon_alloc(size)		malloc(size)
#define mon_free(x)		free(x)

#define MONALLOC_OVERHEAD		16	/* mon_alloc() overhead */
#define MONALLOC_ALIGNMENT		8	/* mon_alloc() alignment */
#endif

#define AR3CMON_NEW_ALIGNMENT_MASK	(~(AR3CMON_NEW_ALIGNMENT - 1))
#define MONALLOC_ALIGNMENT_MASK		(~(MONALLOC_ALIGNMENT - 1))

#define TOKEN_TTL		(3600 * 1000000000ULL)	/* 1h */

#endif
