/*
 * ar3cmonitor - dmeter.cpp
 *
 * Copyright (C) 2006..2011  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <qpainter.h>
#include <cmath>

#include "dmeter.h"
#include "control.h"
#include <cdata.h>
#include <ar3cm_proto.h>

MeterDWidget::MeterDWidget(int _analogIndex, QColor _color, float _unitValue,
		QString _unit, unsigned long flags, QString _prefix,
		QWidget *parent, const char *name, Qt::WFlags f)
	: BufferedWidget(parent, name, f), unit(_unit), prefix(_prefix)
{
	setMinimumWidth(QFontMetrics(CO->valueFont()).width(
				QString("%1-999.99%2").arg(prefix).arg(unit))
			* 6 / 5);
	unit_value = _unitValue;
	color = _color;
	meter_flags = flags;

	vfont = QFont(CO->valueFont());
	setMinimumHeight(QFontMetrics(vfont).height());

	analogIndex = _analogIndex;
	oldvalid = 0;
}

QString MeterDWidget::niceFormat(float value)
{
	return QString().sprintf("%7.2f", value);
}

void MeterDWidget::paint(QPainter *p)
{
	int ret;
	int iv;

	p->setPen(Qt::NoPen);

	if (meter_flags & AR3CM_DMETER_ALONG)
		ret = cdata_get_along(analogIndex, &iv);
	else
		ret = cdata_get_analog(analogIndex, &iv);

	if (ret) {
		p->setPen(Qt::red);
		p->drawText(0, 0, width(), height(), Qt::AlignCenter,
				QString("?"));
		oldvalid = 0;
		return;
	}
	float v = iv / unit_value;

	setMinimumHeight(QFontMetrics(vfont).height());

	p->setPen(color);
	p->setFont(vfont);

	p->drawText(0, 0, width(), height(),
			Qt::AlignHCenter | Qt::AlignCenter | Qt::SingleLine,
			QString("%1%2%3").arg(prefix).arg(niceFormat(v))
			.arg(unit));
}

void MeterDWidget::updateMeter()
{
	int need_update = 0;
	int iv;

	if (cdata_get_analog(analogIndex, &iv)) {
		if (oldvalid)
			update();
		return;
	}

	if (oldvalid) {
		float v = iv / unit_value;

		if (v != old_value)
			need_update = 1;
	} else
		need_update = 1;

	if (need_update)
		update();
}
