//
// C++ Implementation: dzwonekgraph
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2005, 2006  Krzysztof Mazur <krzysiek@podlesie.net>
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qlabel.h>
#include <qpainter.h>

#include "commobj.h"
#include "dzwonekgraph.h"
#include "ar3cmonitor.h"
#include "consts.h"
#include "control.h"

DzwonekGraph::DzwonekGraph(QWidget *parent, const char*name, Qt::WFlags f)
	: BufferedWidget(parent, name, f)
{
	nsigs = 0;
	lineHeight = QFontMetrics(CO->smallFont()).height();
}

void DzwonekGraph::addSignal(int physical, QColor acolor, QColor icolor,
		const QString &symbol)
{
	sigs[nsigs] = Signal(physical, acolor, icolor, symbol);
	nsigs++;
	setMinimumHeight(lineHeight*nsigs);
}

int DzwonekGraph::nextStep()
{
	int i, ret = 0;
	unsigned long long t, ts = sigs[0].iter.timeStamp();
	
	for (i = 1; i < nsigs; i++) {
		t = sigs[i].iter.timeStamp();
		if (t > ts) {
			ret = i;
			ts = t;
		}
	}
	
	return ret;
}

void DzwonekGraph::resetSteps(unsigned long long ts)
{
	int i;
	
	for (i = 0; i < nsigs; i++)
		sigs[i].iter = CO->binary(sigs[i].physical)->Cursor
			.scanLeftBefore(ts);
}

void DzwonekGraph::paint(QPainter * p)
{
	int w = width() - QFontMetrics(CO->smallFont()).width("WW");
	unsigned long long start;
	
	/* 1. Determine range. */
	unsigned long long curts = CO->cursorStamp();
	resetSteps(curts);
	int lastsig = nextStep();
	unsigned long long end = sigs[lastsig].iter.valid() ?
		sigs[lastsig].iter.timeStamp() : 0;

	end += AR3CMON_MAX_DZWONEK_ODSTEP;
	if (end > curts)
		end = curts;
	start = end - AR3CMON_MAX_DZWONEK_SERIA;

	for (int n = 0; n < nsigs; n++) {
		int y = lineHeight * n + lineHeight / 2;

		/* 2. Draw legend. */
		p->setPen(sigs[n].acolor);
		p->setFont(CO->smallFont());
		p->drawText(w + 2, lineHeight * n, width() - w - 2, lineHeight,
				Qt::AlignLeft | Qt::AlignVCenter,
				sigs[n].symbol);

		/* 3. Draw line. */
		if (!CO->haveData()) {
			p->setPen(QPen(sigs[n].icolor, 1, Qt::DashLine));
			p->drawLine(0, y, w, y);
			continue;
		}

		p->setPen(QPen(sigs[n].icolor, 1));
		p->drawLine(0, y, w, y);

		/* 4. Draw signals. */

		int x0 = w;
		TimestampedList<BRecord>::Iterator i =
			CO->binary(sigs[n].physical)->Cursor
			.scanLeftBefore(end);
		for (; i.valid(); --i) {
			unsigned long long ts = i.timeStamp();
			int x = calculateX_bin(ts, start, w, end-start);
			BState s = i.data()->state;
			
			if (s == Undefined) {
				p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv),
							3));
				p->drawLine(x0, y, x == x0 ? x0 + 1 : x, y);
			} else if (s == High) {
				p->setPen(QPen(sigs[n].acolor, 5, Qt::SolidLine, Qt::FlatCap, Qt::BevelJoin));
				p->drawLine(x0, y, x == x0 ? x0 + 1 : x, y);
				//p->drawLine(x0, y - 2, x == x0 ? x0 + 1 : x, y + 2);
			}
			x0 = x;
			if (ts < start)
				break;
		}

		if (x0 > 0) {
			p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv), 5, 
						Qt::SolidLine, Qt::FlatCap,
						Qt::BevelJoin));
			p->drawLine(0, y, x0, y);
		}

		TimestampedList<ARecord>::Iterator it = CO->analog()->Cursor;
		if (!it.valid() || it.timeStamp() < CO->cursorStamp()) {
			int x = calculateX_bin(it.timeStamp(), start, w, end
					- start);
			p->setPen(QPen(QColor(0, 0, 70, QColor::Hsv), 5, 
						Qt::SolidLine, Qt::FlatCap,
						Qt::BevelJoin));
			p->drawLine(w, y, x + 1, y);
		}
	}
}
