/*
 * ar3cmonitor - fonts.c
 *
 * Copyright (C) 2012  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <fonts.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* minimal font size */
#define FONT_SIZE_MIN		6

struct font {
	const char *family;
	unsigned int size;
};

static struct font fonts[AR3CMONITOR_FONT_COUNT];
static int use_pixels;

#if defined(__LCD57__)
#define AR3CMON_FONT_SIZE			1000
#define AR3CMON_SMALL_FONT_SIZE			912
#define AR3CMON_TIMESCALE_FONT_SIZE		833
#define AR3CMON_VALUE_FONT_SIZE			1000
#define AR3CMON_HUGE_FONT_SIZE			3271
#elif defined(__SIMPAD__)
#define AR3CMON_FONT_FAMILY			"unifont"
#define AR3CMON_FONT_SIZE			1440
#define AR3CMON_SMALL_FONT_SIZE			1000
#define AR3CMON_TIMESCALE_FONT_SIZE		1200
#define AR3CMON_VALUE_FONT_SIZE			1788
#define AR3CMON_HUGE_FONT_SIZE			2073
#elif defined(LARGE_FONTS) || defined(TOUCH_SCREEN)
#define AR3CMON_FONT_SIZE			1440
#define AR3CMON_SMALL_FONT_SIZE			1000
#define AR3CMON_TIMESCALE_FONT_SIZE		1000
#define AR3CMON_VALUE_FONT_SIZE			1728
#define AR3CMON_HUGE_FONT_SIZE			3271
#define AR3CMON_FONT_FAMILY			"Bitstream Charter"
#else
#define AR3CMON_FONT_SIZE			1000
#define AR3CMON_SMALL_FONT_SIZE			913
#define AR3CMON_TIMESCALE_FONT_SIZE		833
#define AR3CMON_VALUE_FONT_SIZE			1440
#define AR3CMON_HUGE_FONT_SIZE			2073
#endif

#define AR3CMON_FONT_BASE		AR3CMON_FONT_SIZE

#ifndef AR3CMON_FONT_FAMILY
#define AR3CMON_FONT_FAMILY NULL
#endif

const char *ar3cmon_font_family(enum ar3cmonitor_font font)
{
	return fonts[font].family;
}

unsigned int ar3cmon_font_size(enum ar3cmonitor_font font)
{
	return fonts[font].size;
}

int ar3cmon_font_set_family(enum ar3cmonitor_font font, const char *family)
{
	const char *p = NULL;

	if (family != NULL) {
		p = strdup(family);
		if (p ==  NULL)
			return -ENOMEM;
	}

	free((char *) fonts[font].family);
	fonts[font].family = p;
	return 0;
}

int ar3cmon_font_set_size(enum ar3cmonitor_font font, unsigned int size)
{
	fonts[font].size = size;
	return 0;
}

int ar3cmon_font_set_normsize(enum ar3cmonitor_font font, unsigned int size)
{
	unsigned int base_size;
	
	base_size = ar3cmon_font_size(AR3CMONITOR_FONT_BASE);
	size = (base_size * size + 500) / 1000;
	if (size < FONT_SIZE_MIN)
		size = FONT_SIZE_MIN;
	return ar3cmon_font_set_size(font, size);
}

int ar3cmon_font_set_pixels(int pixels)
{
	use_pixels = pixels;
	return 0;
}

int ar3cmon_font_pixels(void)
{
	return use_pixels;
}

void fonts_reset(void)
{
	unsigned int i;

	for (i = 0; i < AR3CMONITOR_FONT_COUNT; i++)
		ar3cmon_font_set_family(i, AR3CMON_FONT_FAMILY);

	ar3cmon_font_set_normsize(AR3CMONITOR_FONT, AR3CMON_FONT_SIZE);
	ar3cmon_font_set_normsize(AR3CMONITOR_FONT_VALUE,
			AR3CMON_VALUE_FONT_SIZE);
	ar3cmon_font_set_normsize(AR3CMONITOR_FONT_SMALL,
			AR3CMON_SMALL_FONT_SIZE);
	ar3cmon_font_set_normsize(AR3CMONITOR_FONT_TIMESCALE,
			AR3CMON_TIMESCALE_FONT_SIZE);
	ar3cmon_font_set_normsize(AR3CMONITOR_FONT_HUGE,
			AR3CMON_HUGE_FONT_SIZE);
}

void fonts_init(void)
{
	unsigned int i;

	for (i = 0; i < AR3CMONITOR_FONT_COUNT; i++) {
		const char *family = AR3CMON_FONT_FAMILY;

		if (family != NULL)
			family = strdup(family);
		fonts[i].family = family;
	}

	fonts_reset();
}
