/*
 * ar3cmonitor - jumpdlg.cpp
 *
 * 	Jump Dialog
 * 
 * Copyright (C) 2006, 2007  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "ar3cmonitor.h"
#include "consts.h"

#include <qlabel.h>
#include <qpainter.h>

#include "jumpdlg_nodte.h"

JumpDialog::JumpDialog(unsigned long long time, QWidget *parent)
	: QDialog(parent)
{
	cur_time = time / 1000000000ULL;

	vl = new QVBoxLayout(this, 8);

	l = new QLabel(this);
	l->setAlignment(Qt::AlignCenter);
	QFont f(font());
	f.setPointSize(f.pointSize() * 5 / 3);
	l->setFont(f);
	
	char tmp[128];
	struct tm tm;
	
	localtime_r(&cur_time, &tm);
	strftime(tmp, sizeof(tmp), "%d %b %Y  %H:%M:%S", &tm);
	l->setText(QString(tmp));
	vl->addWidget(l);
	l->show();
	
	hl2 = new QHBoxLayout(vl);
	
#define add_button(lo, name, cap)					\
	do {								\
		(name) = new QPushButton((cap), this);			\
		(name)->setMinimumHeight(80);				\
		(name)->setMinimumWidth(100);				\
		lo->addWidget((name));					\
		(name)->show();						\
	} while (0)

	add_button(hl2, mu, tr("m+"));
	add_button(hl2, du, tr("d+"));
	add_button(hl2, hu, tr("h+"));
	add_button(hl2, Mu, tr("M+"));
	add_button(hl2, su, tr("s+"));

	hl3 = new QHBoxLayout(vl);
	add_button(hl3, md, tr("m-"));
	add_button(hl3, dd, tr("d-"));
	add_button(hl3, hd, tr("h-"));
	add_button(hl3, Md, tr("M-"));
	add_button(hl3, sd, tr("s-"));

	connect(mu, SIGNAL(clicked()), this, SLOT(mu_clicked()));
	connect(du, SIGNAL(clicked()), this, SLOT(du_clicked()));
	connect(hu, SIGNAL(clicked()), this, SLOT(hu_clicked()));
	connect(Mu, SIGNAL(clicked()), this, SLOT(Mu_clicked()));
	connect(su, SIGNAL(clicked()), this, SLOT(su_clicked()));

	connect(md, SIGNAL(clicked()), this, SLOT(md_clicked()));
	connect(dd, SIGNAL(clicked()), this, SLOT(dd_clicked()));
	connect(hd, SIGNAL(clicked()), this, SLOT(hd_clicked()));
	connect(Md, SIGNAL(clicked()), this, SLOT(Md_clicked()));
	connect(sd, SIGNAL(clicked()), this, SLOT(sd_clicked()));
	
	hl = new QHBoxLayout(vl);

	ok = new QPushButton(tr("OK"), this);
	ok->setMinimumHeight(60);
	connect(ok, SIGNAL(clicked()), this, SLOT(ok_clicked()));
	hl->addWidget(ok);
	ok->show();
	
	cancel = new QPushButton(tr("Cancel"), this);
	cancel->setMinimumHeight(60);
	connect(cancel, SIGNAL(clicked()), this, SLOT(cancel_clicked()));
	hl->addWidget(cancel);
	cancel->show();
}

void JumpDialog::update_label()
{
	char tmp[128];
	struct tm tm;
	
	localtime_r(&cur_time, &tm);
	strftime(tmp, sizeof(tmp), "%d %b %Y  %H:%M:%S", &tm);
	l->setText(QString(tmp));
}

void JumpDialog::mu_clicked()
{
	struct tm tm;

	localtime_r(&cur_time, &tm);
	tm.tm_mon++;
	cur_time = mktime(&tm);
	update_label();
}

void JumpDialog::du_clicked()
{
	cur_time += 3600 * 24;
	update_label();
}

void JumpDialog::hu_clicked()
{
	cur_time += 3600;
	update_label();
}

void JumpDialog::Mu_clicked()
{
	cur_time += 60;
	update_label();
}

void JumpDialog::su_clicked()
{
	cur_time++;
	update_label();
}

void JumpDialog::md_clicked()
{
	struct tm tm;

	localtime_r(&cur_time, &tm);
	tm.tm_mon--;
	cur_time = mktime(&tm);
	update_label();
	update_label();
}

void JumpDialog::dd_clicked()
{
	cur_time -= 3600 * 24;
	update_label();
}

void JumpDialog::hd_clicked()
{
	cur_time -= 3600;
	update_label();
}

void JumpDialog::Md_clicked()
{
	cur_time -= 60;
	update_label();
}

void JumpDialog::sd_clicked()
{
	cur_time--;
	update_label();
}

void JumpDialog::ok_clicked()
{
	if (!CO->reviewMode())
		CO->enterReviewMode();
	CO->setCursor(cur_time * 1000000000ULL);
	close();
}

void JumpDialog::cancel_clicked()
{
	close();
}

JumpDialog::~JumpDialog()
{
	delete cancel;
	delete ok;
	delete mu;
	delete du;
	delete hu;
	delete Mu;
	delete su;
	delete md;
	delete dd;
	delete hd;
	delete Md;
	delete sd;
	delete hl3;
	delete hl2;
	delete hl;
	delete vl;
}
