/*
 * ar3cmonitor - onstore.cpp
 *
 * 	Online Data Store
 * 
 * Copyright (C) 2006, 2007  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "ar3cmonitor.h"
#include "datastore.h"
#include "offstore.h"
#include "control.h"
#include "commobj.h"
#include <stdio.h>

OnlineStore::OnlineStore(unsigned int binaries, const char *_name)
	: DataStore(binaries, _name)
{
}

void OnlineStore::garbageCollector()
{
	CO->garbageCollector();
}

int OnlineStore::remove_data(unsigned long long margin)
{
	unsigned int i;	
	AnalogControl *a = get_acontrol();
	AList *al = &a->Data;
	Item *item;
	int ret = 0;

	margin = min(margin, CO->timeStamp());
	
	while (al->tmore() && (al->head()->TimeStamp < margin)) {
		item = al->poph();
		
		if (a->Cursor == item)
			++(a->Cursor);

		stat_delete_record(monalloc_tsize(item->get_size()));
		delete item;
	}

	if (al->tmore())
		ret = 1;

	for (i = 0; i < get_binaries(); i++) {
		BinaryControl *b = get_binary(i);
		TimestampedList<BRecord> *bl = &b->Data;

		while (bl->tmore() && (bl->head()->TimeStamp < margin)) {
			item = (BRecord *) bl->poph();

			if (b->Cursor == item)
				++(b->Cursor);
			stat_delete_record(monalloc_tsize(item->get_size()));
			delete item;
		}
	
		if (bl->tmore())
			ret = 1;
	}

	seek_eof();
	return ret;
}

int OnlineStore::shrink(int)
{
	AList::Iterator it = get_acontrol()->Data.it();
	unsigned long long first = (++(++it)).timeStamp(); /* FIXME */
	
	if (!first)
		return 0;
	return remove_data(first + SEC_TS(1));
}

OnlineStore::~OnlineStore()
{
}
