//
// C++ Implementation: statusbar
//
// Description: 
//
//
// Author: Szymon Acedanski <accek@podlesie.net>, (C) 2004
// Copyright (C) 2006  Krzysztof Mazur <krzysiek@podlesie.net>  
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <qlayout.h>
#include <qlabel.h>
#include <time.h>
#include <qcursor.h>

#include "ar3cmonitor.h"
#include "consts.h"
#include "commobj.h"
#include "control.h"
#include "onoffwidget.h"
#include "statusbar.h"

#if QT_VERSION >= 0x40100
# define setAutoFillBG(x)	((x)->setAutoFillBackground(true))
#else
# define setAutoFillBG(x)	do { } while (0)
#endif

MainStatusBar::MainStatusBar(QWidget * parent, const char *name)
	: QWidget(parent, name)
{
	QFont f;

#ifdef AR3CMON_STATUSBAR_FONT_SIZE
	f.setPointSize(AR3CMON_STATUSBAR_FONT_SIZE);
#endif

	QHBoxLayout *l = new QHBoxLayout(this, 2);

	ConnectedLabel = new OnOffLabel(trUtf8("CONNECTED"), Qt::green,
			QColor(50, 0, 0), this);
	ConnectedLabel->setFont(f);
	setAutoFillBG(ConnectedLabel);
	DataLabel = new OnOffLabel(tr("NO TRANSMISSION"), Qt::red,
			QColor(50, 0, 0), this);
	DataLabel->setFont(f);
	setAutoFillBG(DataLabel);
	ModeLabel = new OnOffLabel(tr(""), Qt::white, QColor(50, 50, 50), this);
	ModeLabel->setFont(f);
	setAutoFillBG(ModeLabel);

	conn = new QPopupMenu();
	conn->insertItem(tr("&Connect"), TheCommObject, SLOT(connectToAr3c()));
	conn->insertItem(trUtf8("&Disconnect"), TheCommObject, SLOT(goOffline()));
	connect(ConnectedLabel, SIGNAL(clicked()), this, SLOT(change_conn()));

	mode = new QPopupMenu();
	mode->insertItem(trUtf8("&Online"), CO, SLOT(enterOnlineMode()));
	mode->insertItem(trUtf8("&Review"), CO, SLOT(enterReviewMode()));
	mode->insertItem(trUtf8("&Offline"), CO, SLOT(enterOfflineMode()));
	connect(ModeLabel, SIGNAL(clicked()), this, SLOT(change_mode()));

	DataBlinker = new QTimer(this);
	ModeBlinker = new QTimer(this);
	DataBlink = false;
	ModeBlink = false;

	SyfLabel1 = new OnOffLabel(trUtf8("01 Jan 1970 00:00:00"), Qt::yellow, Qt::black,
			this);
	SyfLabel1->setFont(f);
	setAutoFillBG(SyfLabel1);
#ifdef __ARM__
	SyfLabel1->setMinimumWidth(150);
#else
	SyfLabel1->setMinimumWidth(SyfLabel1->
			fontMetrics().width(trUtf8("01 AAA 2222 00:00:00")) + 40);
#endif
	SyfLabel1->off();
	l->addWidget(SyfLabel1);

	l->addWidget(ModeLabel);
	l->addWidget(DataLabel);
	l->addWidget(ConnectedLabel);
	ModeLabel->on();
	ModeLabel->show();
	DataLabel->show();
	ConnectedLabel->show();
	onModeChanged();

#ifdef __ARM__
	ConnectedLabel->setMinimumWidth(150);
	ModeLabel->setMinimumWidth(150);
	DataLabel->setMinimumWidth(150);
	ConnectedLabel->hide();
	DataLabel->hide();
#else
	ConnectedLabel->setMinimumWidth(ConnectedLabel->
			fontMetrics().width(ConnectedLabel->text()) + 20);
	ModeLabel->setMinimumWidth(ModeLabel->
			fontMetrics().width(tr("INITIALIZATION")) + 20);
	DataLabel->setMinimumWidth(DataLabel->
			fontMetrics().width(DataLabel->text()) + 20);
#endif

	connect(CO, SIGNAL(sigHaveDataChanged()), SLOT(dataUpdate()));
	connect(CO, SIGNAL(sigModeChanged()), SLOT(onModeChanged()));
	connect(CO, SIGNAL(sigModeChanged()), SLOT(dataUpdate()));
	connect(CO, SIGNAL(sigDownloadingOrNot()), SLOT(onModeChanged()));
	connect(TheCommObject, SIGNAL(sigError()), SLOT(onModeChanged()));
	connect(TheCommObject, SIGNAL(sigFullyConnected()),
			SLOT(onModeChanged()));
	connect(TheCommObject, SIGNAL(sigError()), SLOT(dataUpdate()));
	connect(TheCommObject, SIGNAL(sigFullyConnected()), SLOT(dataUpdate()));
	connect(TheCommObject, SIGNAL(sigError()), ConnectedLabel, SLOT(off()));
	connect(TheCommObject, SIGNAL(sigConnected()),
			ConnectedLabel, SLOT(on()));
	connect(DataBlinker, SIGNAL(timeout()), SLOT(dataBlink()));
	connect(ModeBlinker, SIGNAL(timeout()), SLOT(modeBlink()));
}

void MainStatusBar::onModeChanged()
{
	if (!TheCommObject->fullyConnected()) {
		ModeLabel->setText(trUtf8("INITIALIZATION"));
		ModeBlinker->stop();
		ModeLabel->on();
	} else if (CO->offlineMode()) {
		ModeLabel->setText(trUtf8("OFFLINE"));
		if (CO->offlineDownloading()) {
			if (!ModeBlinker->isActive()) {
				ModeBlink = false;
				modeBlink();
			}
		} else {
			ModeBlinker->stop();
			ModeLabel->on();
		}
	} else if (CO->reviewMode()) {
		ModeLabel->setText(trUtf8("REVIEW"));
		ModeBlinker->stop();
		ModeLabel->on();
	} else {
		ModeLabel->setText(trUtf8("ONLINE"));
		ModeBlinker->stop();
		ModeLabel->on();
	}
}

void MainStatusBar::dataUpdate()
{
	if (CO->realHaveData()) {
		DataBlinker->stop();
		DataLabel->off();
	} else {
		DataLabel->on();
		if (TheCommObject->fullyConnected() &&
			TheCommObject->reallyConnected()) {
			if (!DataBlinker->isActive()) {
				DataBlink = true;
				dataBlink();
			}
		} else
			DataBlinker->stop();
	}
}

void MainStatusBar::dataBlink()
{
	if (DataBlink) {
		DataBlink = false;
		DataLabel->on();
		DataBlinker->start(200, TRUE);
	} else {
		DataBlink = true;
		DataLabel->off();
		DataBlinker->start(800, TRUE);
	}
}

void MainStatusBar::modeBlink()
{
	if (ModeBlink) {
		ModeBlink = false;
		ModeLabel->on();
		ModeBlinker->start(200, TRUE);
	} else {
		ModeBlink = true;
		ModeLabel->off();
		ModeBlinker->start(800, TRUE);
	}
}

void MainStatusBar::newTimestamp(QString time)
{
	if (!SyfLabel1)
		return;

	SyfLabel1->setText(time);
//	DataLabel->setActiveColor(Qt::yellow);
	SyfLabel1->on();
}

void MainStatusBar::change_mode()
{
	unsigned int size = mode->sizeHint().height();

	mode->exec(ModeLabel->mapToGlobal(QPoint(0, -size)));
}

void MainStatusBar::change_conn()
{
	unsigned int size = conn->sizeHint().height();
	conn->exec(ConnectedLabel->mapToGlobal(QPoint(0, -size)));
}
